//
//  ViewController.m
//  Test
//
//  Created by 余西安 on 2016/10/8.
//  Copyright © 2016年 余西安. All rights reserved.
//

#import "ViewController.h"

/************************ SACollectionViewLayout *****************************/

@interface SACollectionViewLayout : UICollectionViewFlowLayout

@end

@implementation SACollectionViewLayout

- (nullable NSArray<__kindof UICollectionViewLayoutAttributes *> *)layoutAttributesForElementsInRect:(CGRect)rect
{
    NSArray *array = [super layoutAttributesForElementsInRect:rect];
    if (array.count == 0) return array;
    NSInteger count = array.count;
    NSMutableArray *attributes = [NSMutableArray array];
    [attributes addObject:[array firstObject]];
    for (int i = 0; i < count; i++) {
        if (i == 0) continue;
        UICollectionViewLayoutAttributes *a0 = [array objectAtIndex:i - 1];
        UICollectionViewLayoutAttributes *a1 = [array objectAtIndex:i];
        CGRect frame0 = a0.frame;
        CGRect frame1 = a1.frame;
        // 当前这个不为行内首个时才需要修改frame
        if (frame0.origin.y == frame1.origin.y) {
            frame1.origin.x = CGRectGetMaxX(frame0) + 10;
            a1.frame = frame1;
        }
        [attributes addObject:a1];
    }
    return attributes;
}

@end

/****************************** ViewController **********************************/

@interface ViewController () <UICollectionViewDataSource, UICollectionViewDelegate>

@property (nonatomic, strong) UICollectionView *collectionView;

@end

@implementation ViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    SACollectionViewLayout *layout = [[SACollectionViewLayout alloc] init];
    layout.itemSize = CGSizeMake(100, 100);
    layout.sectionInset = UIEdgeInsetsMake(20, 8, 20, 8);
    layout.scrollDirection = UICollectionViewScrollDirectionVertical;
    CGRect frame = self.view.bounds;
    self.collectionView = [[UICollectionView alloc] initWithFrame:frame collectionViewLayout:layout];
    self.collectionView.dataSource = self;
    self.collectionView.delegate = self;
    self.collectionView.backgroundColor = [UIColor whiteColor];
    [self.collectionView registerClass:[UICollectionViewCell class] forCellWithReuseIdentifier:@"SACollectionViewCell"];
    [self.view addSubview:self.collectionView];
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    return 120;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    UICollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"SACollectionViewCell" forIndexPath:indexPath];
    cell.contentView.layer.borderColor = [[UIColor lightGrayColor] CGColor];
    cell.contentView.backgroundColor = [UIColor colorWithWhite:0.9 alpha:1.0];
    cell.contentView.layer.borderWidth = 0.5f;
    cell.contentView.layer.cornerRadius = 15.0f;
    return cell;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath
{
    CGFloat s = arc4random_uniform(5);
    CGFloat width = 45 + s * 5 ;
    CGFloat height = 30;
    return CGSizeMake(width, height);
}

@end
